<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\FoodOrder;
use App\Entity\ItemExtra;
use App\Entity\MenuItem;
use App\Entity\MenuItemAlias;
use App\Entity\OrderItem;
use App\Form\ItemExtraType;
use App\Form\MenuItemType;
use App\Form\OrderItemType;
use App\Repository\ItemExtraRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/order')]
class FoodOrderController extends AbstractController
{
    final public const APP_FOOD_ORDER_SHOW = 'app_food_order_show';

    final public const APP_FOODORDER_ADD_ITEM = 'app_foodorder_add_item';

    final public const APP_FOODORDER_CLOSE = 'app_foodorder_close';

    final public const APP_FOODORDER_COPY_ITEM = 'app_foodorder_item_copy';

    final public const APP_FOODORDER_REMOVE_ITEM = 'app_foodorder_item_remove';

    #[Route(
        '/{id}',
        name: self::APP_FOOD_ORDER_SHOW,
        methods: [Request::METHOD_GET, Request::METHOD_POST],
    )]
    public function show(
        FoodOrder $foodOrder,
        Request $request,
        EntityManagerInterface $entityManager,
    ): Response {
        $newItem = new MenuItem;
        $newItem->setVendor($foodOrder->getVendor());
        $newItem->setPrice(0);
        $newItemForm = $this->createForm(MenuItemType::class, $newItem);
        $newItemForm->handleRequest($request);

        if ($newItemForm->isSubmitted() && $newItemForm->isValid()) {
            $name = (string) $newItemForm->get('name')
                ->getData();
            $alias = new MenuItemAlias;
            $alias->setName($name);
            $alias->setMenuItem($newItem);
            $entityManager->persist($newItem);
            $entityManager->persist($alias);
            $entityManager->flush();
            return $this->redirectToRoute(
                self::APP_FOOD_ORDER_SHOW,
                [
                    'id' => $foodOrder->getId(),
                ],
            );
        }

        return $this->render(
            'food_order/show.html.twig',
            [
                'food_order' => $foodOrder,
                'menu_items' => $foodOrder->getVendor()
                    ->getMenuItems(),
                'form' => $newItemForm,
            ],
        );
    }

    #[Route('/close/{id}', name: self::APP_FOODORDER_CLOSE, methods: ['GET'])]
    public function close(
        FoodOrder $foodOrder,
        EntityManagerInterface $entityManager,
    ): Response {
        $foodOrder->setClosedAt();
        $entityManager->persist($foodOrder);
        $entityManager->flush();
        return $this->redirectToRoute(
            self::APP_FOOD_ORDER_SHOW,
            [
                'id' => $foodOrder->getId(),
            ],
            Response::HTTP_SEE_OTHER,
        );
    }

    #[Route(
        '/{foodOrder}/add/{menuItem}',
        name: self::APP_FOODORDER_ADD_ITEM,
        methods: [Request::METHOD_GET, Request::METHOD_POST],
    )]
    public function add(
        FoodOrder $foodOrder,
        MenuItem $menuItem,
        EntityManagerInterface $entityManager,
        ItemExtraRepository $extraRepository,
        Request $request,
    ): Response {
        $orderItem = new OrderItem;
        $orderItem->setFoodOrder($foodOrder);
        $orderItem->setMenuItem($menuItem);
        $form = $this->createForm(
            OrderItemType::class,
            $orderItem,
            [
                'menuItem' => $menuItem,
            ],
        );
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $entityManager->persist($orderItem);
            $entityManager->flush();
            return $this->redirectToRoute(
                self::APP_FOOD_ORDER_SHOW,
                [
                    'id' => $foodOrder->getId(),
                ],
                Response::HTTP_SEE_OTHER,
            );
        }

        $newExtra = new ItemExtra;
        $newExtra->setMenuItem($menuItem);
        $extraForm = $this->createForm(
            ItemExtraType::class,
            $newExtra,
            [
                'menuItem' => $menuItem,
            ],
        );
        $extraForm->handleRequest($request);

        if ($extraForm->isSubmitted() && $extraForm->isValid()) {
            $entityManager->persist($newExtra);
            $entityManager->flush();
            return $this->redirectToRoute(
                self::APP_FOODORDER_ADD_ITEM,
                [
                    'foodOrder' => $foodOrder->getId(),
                    'menuItem' => $menuItem->getId(),
                ],
                Response::HTTP_SEE_OTHER,
            );
        }

        return $this->render(
            'food_order/orderitem.html.twig',
            [
                'form' => $form,
                'extraForm' => $extraForm,
                'extras' => $extraRepository->getUniqueNames(),
            ],
        );
    }

    #[Route(
        '/item/remove/{orderItem}',
        name: self::APP_FOODORDER_REMOVE_ITEM,
        methods: [Request::METHOD_GET],
    )]
    public function remove(
        OrderItem $orderItem,
        EntityManagerInterface $entityManager,
    ): Response {
        $orderId = $orderItem->getFoodOrder()
            ->getId();
        $entityManager->remove($orderItem);
        $entityManager->flush();
        return $this->redirectToRoute(
            self::APP_FOOD_ORDER_SHOW,
            [
                'id' => $orderId,
            ],
            Response::HTTP_SEE_OTHER,
        );
    }

    #[Route(
        '/item/copy/{orderItem}',
        name: self::APP_FOODORDER_COPY_ITEM,
        methods: [Request::METHOD_GET],
    )]
    public function copyItem(
        OrderItem $orderItem,
        EntityManagerInterface $entityManager,
    ): Response {
        $newItem = new OrderItem;
        $newItem->setFoodOrder($orderItem->getFoodOrder());
        $newItem->setMenuItem($orderItem->getMenuItem());

        foreach ($orderItem->getExtras() as $extra) {
            $newItem->getExtras()
                ->add($extra);
        }

        $entityManager->persist($newItem);
        $entityManager->flush();
        return $this->redirectToRoute(
            self::APP_FOOD_ORDER_SHOW,
            [
                'id' => $orderItem->getFoodOrder()
                    ->getId(),
            ],
        );
    }
}
